#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <netinet/in.h>
#include <netinet/ip_icmp.h>
#include <linux/if.h>
#include <linux/netlink.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_tcp.h>
#include <linux/netfilter_ipv4/ip_conntrack_icmp.h>
#include <linux/netfilter_ipv4/ip_conntrack_ftp.h>
#include <linux/netfilter_ipv4/ip_conntrack_netlink.h>

#include "utility.h"
#include "display.h"

char *pr_ctinfo(unsigned short info)
{
        static char buff[250];
        int len = 0;

        memset(buff, '\0', sizeof(buff));
        switch(CTINFO2DIR(info)) {
                case (IP_CT_DIR_REPLY):
                        len += sprintf(buff + len, "Reply");
                        break;
                case (IP_CT_DIR_ORIGINAL):
                        len += sprintf(buff + len, "Original");
                        break;
                default:
                        len += sprintf(buff + len, "Unknown %d",
                                CTINFO2DIR(info));
                        break;
        }

        len += sprintf(buff + len, ", ");
        switch(info) {
                case (IP_CT_NEW):
                        len += sprintf(buff + len, "New");
                        break;
                case (IP_CT_ESTABLISHED):
                        len += sprintf(buff + len, "Established");
                        break;
                case (IP_CT_RELATED):
                        len += sprintf(buff + len, "Related");
                        break;
                case (IP_CT_IS_REPLY):
                        len += sprintf(buff + len, "Reply");
                        break;
                case (IP_CT_DELETE):
                        len += sprintf(buff + len, "Delete");
                        break;
                default:
                        len += sprintf(buff + len, "Unknown %d", info);
                        break;
        }

        return (buff);
}

char *display_sockaddr_nl(struct sockaddr_nl *nl)
{
        static char buff[250];

        memset(buff, '\0', sizeof(buff));
        sprintf(buff, "Family (%s) Pid (%d) Groups (%02X)",
                pr_af_family(nl->nl_family), nl->nl_pid, nl->nl_groups);

        return (buff);
}

char *pr_nlmsg_type(unsigned short type)
{
        static char buff[20];

        memset(buff, '\0', sizeof(buff));
        switch(type) {
                case (NLMSG_NOOP):
                        sprintf(buff, "No operation");
                        break;
                case (NLMSG_ERROR):
                        sprintf(buff, "Error");
                        break;
                case (NLMSG_DONE):
                        sprintf(buff, "Done");
                        break;
                case (NLMSG_OVERRUN):
                        sprintf(buff, "Overrun");
                        break;
                default:
                        sprintf(buff, "Unknown (%02X)", type);
                        break;
        }

        return (buff);
}

char *display_nlmsg(struct nlmsghdr *nl)
{
        static char buff[100];

        memset(buff, '\0', sizeof(buff));
        sprintf(buff, "Length (%d) Type (%s) Flags (%02X) "
                "Sequence (%d) Pid (%d)",
                nl->nlmsg_len, pr_nlmsg_type(nl->nlmsg_type), nl->nlmsg_flags,
                nl->nlmsg_seq, nl->nlmsg_pid);

        return (buff);
}

char *display_nlmsg_error(struct nlmsgerr *nlerr, int msg_len)
{
        static char buff[50];
        int len;

        memset(buff, '\0', sizeof(buff));
        len = sprintf(buff, "%s", "payload contains error data: ");
        if(msg_len < NLMSG_LENGTH(sizeof(struct nlmsgerr)))
                len += sprintf(buff + len, "error packet is truncated.");
        else
        {
                errno = -nlerr->error;
                len += sprintf(buff + len, "%s.", strerror(errno));
        }

        return (buff);
}

char *display_tuple_flat(struct ip_conntrack_tuple *tuple)
{
        static char buff[250];
        char psb[20];
        int len = 0;

        memset(buff, '\0', sizeof(buff));
	len += sprintf(buff + len, "%s:", in_ntoa(tuple->src.ip));
        switch(tuple->dst.protonum) {
                case (IPPROTO_ICMP):
                        len += sprintf(buff + len, "Icmp (id %d)",
                                ntohs(tuple->src.u.icmp.id));
                        break;
                case (IPPROTO_TCP):
                        sprintf(psb, "%d", ntohs(tuple->src.u.tcp.port));
                        len += sprintf(buff + len, "%s", psb);
                        break;
                case (IPPROTO_UDP):
                        sprintf(psb, "%d", ntohs(tuple->src.u.udp.port));
                        len += sprintf(buff + len, "%s", psb);
                        break;
                default:
                        len += sprintf(buff + len, "Unknown");
                        break;
        }

	len += sprintf(buff + len, "->");
        len += sprintf(buff + len, "%s:", in_ntoa(tuple->dst.ip));
        switch(tuple->dst.protonum) {
                case (IPPROTO_ICMP):
                        len += sprintf(buff + len, "Icmp (%s, code %d)",
                                pr_icmp_type(tuple->dst.u.icmp.type),
                                tuple->dst.u.icmp.code);
                        break;
                case (IPPROTO_TCP):
                        sprintf(psb, "%d", ntohs(tuple->dst.u.tcp.port));
                        len += sprintf(buff + len, "%s", psb);
                        break;
                case (IPPROTO_UDP):
                        sprintf(psb, "%d", ntohs(tuple->dst.u.udp.port));
                        len += sprintf(buff + len, "%s", psb);
                        break;
                default:
                        len += sprintf(buff + len, "Unknown");
                        break;
        }

        return (buff);
}

char *pr_tcp_state(unsigned char state)
{
        static char buff[25];

        memset(buff, '\0', sizeof(buff));
        switch(state) {
                case (TCP_CONNTRACK_NONE):
                        sprintf(buff, "none");
                        break;
                case (TCP_CONNTRACK_ESTABLISHED):
                        sprintf(buff, "established");
                        break;
                case (TCP_CONNTRACK_SYN_SENT):
                        sprintf(buff, "syn sent");
                        break;
                case (TCP_CONNTRACK_SYN_RECV):
                        sprintf(buff, "syn received");
                        break;
                case (TCP_CONNTRACK_FIN_WAIT):
                        sprintf(buff, "fin wait");
                        break;
                case (TCP_CONNTRACK_TIME_WAIT):
                        sprintf(buff, "time wait");
                        break;
                case (TCP_CONNTRACK_CLOSE):
                        sprintf(buff, "close");
                        break;
                case (TCP_CONNTRACK_CLOSE_WAIT):
                        sprintf(buff, "close wait");
                        break;
                case (TCP_CONNTRACK_LAST_ACK):
                        sprintf(buff, "last ack");
                        break;
                case (TCP_CONNTRACK_LISTEN):
                        sprintf(buff, "listen");
                        break;
                default:
                        sprintf(buff, "unknown");
                        break;
        }

        return (buff);
}

char *pr_icmp_type(unsigned char type)
{
        static char buff[25];

        memset(buff, '\0', sizeof(buff));
        switch(type) {
                case (ICMP_ECHOREPLY):
                        sprintf(buff, "%s", "Echo Reply");
                        break;
                case (ICMP_DEST_UNREACH):
                        sprintf(buff, "%s", "Destination Unreachable");
                        break;
                case (ICMP_SOURCE_QUENCH):
                        sprintf(buff, "%s", "Source Quench");
                        break;
                case (ICMP_REDIRECT):
                        sprintf(buff, "%s", "Redirect");
                        break;
                case (ICMP_ECHO):
                        sprintf(buff, "%s", "Echo Request");
                        break;
                case (ICMP_TIME_EXCEEDED):
                        sprintf(buff, "%s", "Time Exceeded");
                        break;
                case (ICMP_PARAMETERPROB):
                        sprintf(buff, "%s", "Parameter Problem");
                        break;
                case (ICMP_TIMESTAMP):
                        sprintf(buff, "%s", "Timestamp Request");
                        break;
                case (ICMP_TIMESTAMPREPLY):
                        sprintf(buff, "%s", "Timestamp Reply");
                        break;
                case (ICMP_INFO_REQUEST):
                        sprintf(buff, "%s", "Information Request");
                        break;
                case (ICMP_INFO_REPLY):
                        sprintf(buff, "%s", "Information Reply");
                        break;
                case (ICMP_ADDRESS):
                        sprintf(buff, "%s", "Address Mask Request");
                        break;
                case (ICMP_ADDRESSREPLY):
                        sprintf(buff, "%s", "Address Mask Reply");
                        break;
                default:
                        sprintf(buff, "%s", "unknown");
                        break;
        }

        return (buff);
}

char *pr_ct_status(unsigned long status)
{
        static char buf[100];

        if(!(status & IPS_SEEN_REPLY))
                sprintf(buf, "[UNREPLIED] ");
        if(status & IPS_ASSURED)
                sprintf(buf, "[ASSURED] ");
        return (buf);
}

char *pr_man_type(unsigned short m)
{
        static char buf[10];

        switch(m) {
                case (IP_NAT_MANIP_SRC):
                        sprintf(buf, "Src");
                        break;

                case (IP_NAT_MANIP_DST):
                        sprintf(buf, "Dst");
                        break;
        }

        return (buf);
}

char *pr_man_direction(unsigned char d)
{
        static char buf[10];

        switch(d) {
                case (IP_CT_DIR_REPLY):
                        sprintf(buf, "Rply");
                        break;
                case (IP_CT_DIR_ORIGINAL):
                        sprintf(buf, "Orig");
                        break;
        }

        return (buf);
}

char *pr_proto(int p)
{
        static char buff[25];

        memset(buff, '\0', sizeof(buff));
        switch (p) {
                case (0):
                        sprintf(buff, "Any");
                        break;
                case (IPPROTO_ICMP):
                        sprintf(buff, "Icmp");
                        break;
                case (IPPROTO_IGMP):
                        sprintf(buff, "Igmp");
                        break;
                case (IPPROTO_IPIP):
                        sprintf(buff, "Ipip");
                        break;
                case (IPPROTO_TCP):
                        sprintf(buff, "Tcp");
                        break;
                case (IPPROTO_EGP):
                        sprintf(buff, "Egp");
                        break;
                case (IPPROTO_PUP):
                        sprintf(buff, "Pup");
                        break;
                case (IPPROTO_UDP):
                        sprintf(buff, "Udp");
                        break;
                case (IPPROTO_IDP):
                        sprintf(buff, "Idp");
                        break;
                case (IPPROTO_RSVP):
                        sprintf(buff, "Rsvp");
                        break;
                case (IPPROTO_GRE):
                        sprintf(buff, "Gre");
                        break;
                case (IPPROTO_IPV6):
                        sprintf(buff, "Ipv6");
                        break;
                case (IPPROTO_PIM):
                        sprintf(buff, "Pim");
                        break;
                case (IPPROTO_ESP):
                        sprintf(buff, "Esp");
                        break;
                case (IPPROTO_AH):
                        sprintf(buff, "Ah");
                        break;
                case (IPPROTO_COMP):
                        sprintf(buff, "Comp");
                        break;
                case (IPPROTO_RAW):
                        sprintf(buff, "Raw");
                        break;
                default:
                        sprintf(buff, "Unknown");
                        break;
        }

        return (buff);
}

char *pr_af_family(unsigned char family)
{
        static char buff[30];

        memset(buff, '\0', sizeof(buff));
        switch(family) {
                case (AF_LOCAL):
                        sprintf(buff, "AF_LOCAL");
                        break;
                case (AF_INET):
                        sprintf(buff, "IPv4");
                        break;
                case (AF_AX25):
                        sprintf(buff, "AX.25");
                        break;
                case (AF_IPX):
                        sprintf(buff, "IPX");
                        break;
                case (AF_APPLETALK):
                        sprintf(buff, "Appletalk");
                        break;
                case (AF_NETROM):
                        sprintf(buff, "AF_NETROM");
                        break;
                case (AF_BRIDGE):
                        sprintf(buff, "AF_BRIDGE");
                        break;
                case (AF_ATMPVC):
                        sprintf(buff, "AF_ATMPVC");
                        break;
                case (AF_X25):
                        sprintf(buff, "X.25");
                        break;
                case (AF_INET6):
                        sprintf(buff, "IPv6");
                        break;
                case (AF_ROSE):
                        sprintf(buff, "AF_ROSE");
                        break;
                case (AF_DECnet):
                        sprintf(buff, "DECnet");
                        break;
                case (AF_NETBEUI):
                        sprintf(buff, "AF_NETBEUI");
                        break;
                case (AF_SECURITY):
                        sprintf(buff, "AF_SECURITY");
                        break;
                case (AF_KEY):
                        sprintf(buff, "AF_KEY");
                        break;
                case (AF_NETLINK):
                        sprintf(buff, "AF_NETLINK");
                        break;
                case (AF_PACKET):
                        sprintf(buff, "AF_PACKET");
                        break;
                case (AF_ASH):
                        sprintf(buff, "AF_ASH");
                        break;
                case (AF_ECONET):
                        sprintf(buff, "AF_ECONET");
                        break;
                case (AF_ATMSVC):
                        sprintf(buff, "AF_ATMSVC");
                        break;
                case (AF_SNA):
                        sprintf(buff, "AF_SNA");
                        break;
                case (AF_IRDA):
                        sprintf(buff, "AF_IRDA");
                        break;
                case (AF_PPPOX):
                        sprintf(buff, "AF_PPPOX");
                        break;
                default:
                        sprintf(buff, "Unknown (%d)", family);
        }

        return (buff);
}
