/* libctnetlink.h: Header file for the Connection Tracking library.
 *
 * Jay Schulist <jschlst@samba.org>, Copyright (c) 2001.
 *
 * This software may be used and distributed according to the terms
 * of the GNU General Public License, incorporated herein by reference.
 */

#ifndef __LIBCTNETLINK_H
#define __LIBCTNETLINK_H

#include <netinet/in.h>
#include <asm/types.h>
#include <linux/if.h>
#include <linux/netlink.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_tcp.h>
#include <linux/netfilter_ipv4/ip_conntrack_ftp.h>
#include <linux/netfilter_ipv4/ip_conntrack_icmp.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_netlink.h> 

#define CTNL_BUFFSIZE	8192

struct ctnl_handle {
	int 			fd;
	struct sockaddr_nl      local;
        struct sockaddr_nl      peer;
        u_int32_t               seq;
        u_int32_t               dump;
};

extern int ctnl_open(struct ctnl_handle *cth, unsigned subscriptions);
extern int ctnl_close(struct ctnl_handle *cth);
extern int ctnl_send(struct ctnl_handle *cth, char *buf, int len);
extern int ctnl_listen(struct ctnl_handle *ctnl,
        int (*handler)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
        void *jarg);
extern int ctnl_talk(struct ctnl_handle *ctnl, struct nlmsghdr *n, pid_t peer,
        unsigned groups, struct nlmsghdr *answer,
        int (*junk)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
	void *jarg);
extern int ctnl_wilddump_request(struct ctnl_handle *cth, int family, int type);
extern int ctnl_dump_request(struct ctnl_handle *cth, int type, void *req, 
	int len);
extern int ctnl_dump_filter(struct ctnl_handle *cth,
        int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
        void *arg1,
        int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
        void *arg2);
extern int ctnl_addattr32(struct nlmsghdr *n, int maxlen, int type, 
	u_int32_t data);
extern int ctnl_addattr_l(struct nlmsghdr *n, int maxlen, int type, void *data,
        int alen);
extern int ctnl_cta_addattr32(struct ctattr *cta, int maxlen, int type, 
	u_int32_t data);
extern int ctnl_cta_addattr_l(struct ctattr *cta, int maxlen, int type,
        void *data, int alen);
extern int ctnl_parse_attr(struct ctattr *tb[], int max, struct ctattr *cta, 
	int len);

#endif	/* __LIBCTNETLINK_H */
